/**
    Copyright 2017 Andrea "Stock" Stocchero

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

	    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
*/
package org.pepstock.charba.client.enums;

import org.pepstock.charba.client.commons.Key;

/**
 * Controls the ticks generation on cartesian time axis.
 * 
 * @author Andrea "Stock" Stocchero
 */
public enum TickSource implements Key
{
	/**
	 * Generates "optimal" ticks based on scale size and time options.
	 */
	AUTO("auto"),
	/**
	 * Generates ticks from data (including labels from data <code>{t|x|y}</code> objects).
	 */
	DATA("data"),
	/**
	 * Generates ticks from user given <code>data.labels</code> values ONLY.
	 */
	LABELS("labels");

	// name value of property
	private final String value;

	/**
	 * Creates with the property value to use in the native object.
	 * 
	 * @param value value of property name
	 */
	private TickSource(String value) {
		this.value = value;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.pepstock.charba.client.commons.Key#value()
	 */
	@Override
	public String value() {
		return value;
	}

}
